"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupWindowsEnvironmentPath = void 0;
const error_1 = __importDefault(require("@pnpm/error"));
const path_1 = require("path");
const execa_1 = __importDefault(require("execa"));
const errors_1 = require("./errors");
const REG_KEY = 'HKEY_CURRENT_USER\\Environment';
async function setupWindowsEnvironmentPath(pnpmHomeDir, opts) {
    // Use `chcp` to make `reg` use utf8 encoding for output.
    // Otherwise, the non-ascii characters in the environment variables will become garbled characters.
    const chcpResult = await (0, execa_1.default)('chcp');
    const cpMatch = /\d+/.exec(chcpResult.stdout) ?? [];
    const cpBak = parseInt(cpMatch[0]);
    if (chcpResult.failed || !(cpBak > 0)) {
        return `exec chcp failed: ${cpBak}, ${chcpResult.stderr}`;
    }
    await (0, execa_1.default)('chcp', ['65001']);
    try {
        return await _setupWindowsEnvironmentPath(path_1.win32.normalize(pnpmHomeDir), opts);
    }
    finally {
        await (0, execa_1.default)('chcp', [cpBak.toString()]);
    }
}
exports.setupWindowsEnvironmentPath = setupWindowsEnvironmentPath;
async function _setupWindowsEnvironmentPath(pnpmHomeDir, opts) {
    const logger = [];
    logger.push(logEnvUpdate(await updateEnvVariable('PNPM_HOME', pnpmHomeDir, opts), 'PNPM_HOME'));
    logger.push(logEnvUpdate(await prependToPath('%PNPM_HOME%'), 'Path'));
    return logger.join('\n');
}
function logEnvUpdate(envUpdateResult, envName) {
    switch (envUpdateResult) {
        case 'skipped': return `${envName} was already up-to-date`;
        case 'updated': return `${envName} was updated`;
    }
    return '';
}
async function updateEnvVariable(name, value, opts) {
    const currentValue = await getEnvValueFromRegistry(name);
    if (currentValue && !opts.force) {
        if (currentValue !== value) {
            throw new errors_1.BadEnvVariableError({ envName: name, currentValue, wantedValue: value });
        }
        return 'skipped';
    }
    else {
        await setEnvVarInRegistry(name, value);
        return 'updated';
    }
}
async function prependToPath(prependDir) {
    const pathData = await getEnvValueFromRegistry('Path');
    if (pathData === undefined || pathData == null || pathData.trim() === '') {
        throw new error_1.default('NO_PATH', '"Path" environment variable is not found in the registry');
    }
    else if (pathData.split(path_1.win32.delimiter).includes(prependDir)) {
        return 'skipped';
    }
    else {
        const newPathValue = `${prependDir}${path_1.win32.delimiter}${pathData}`;
        await setEnvVarInRegistry('Path', newPathValue);
        return 'updated';
    }
}
async function getEnvValueFromRegistry(envVarName) {
    const queryResult = await (0, execa_1.default)('reg', ['query', REG_KEY, '/v', envVarName], { windowsHide: false });
    if (queryResult.failed) {
        throw new error_1.default('REG_READ', 'Win32 registry environment values could not be retrieved');
    }
    const regexp = new RegExp(`^ {4}(?<name>${envVarName}) {4}(?<type>\\w+) {4}(?<data>.*)$`, 'gim');
    const match = Array.from(queryResult.stdout.matchAll(regexp))[0];
    return match?.groups.data;
}
async function setEnvVarInRegistry(envVarName, envVarValue) {
    // `windowsHide` in `execa` is true by default, which will cause `chcp` to have no effect.
    const addResult = await (0, execa_1.default)('reg', ['add', REG_KEY, '/v', envVarName, '/t', 'REG_EXPAND_SZ', '/d', envVarValue, '/f'], { windowsHide: false });
    if (addResult.failed) {
        throw new error_1.default('FAILED_SET_ENV', `Failed to set "${envVarName}" to "${envVarValue}": ${addResult.stderr}`);
    }
    else {
        // When setting environment variables through the registry, they will not be recognized immediately.
        // There is a workaround though, to set at least one environment variable with `setx`.
        // We have some redundancy here because we run it for each env var.
        // It would be enough also to run it only for the last changed env var.
        // Read more at: https://bit.ly/39OlQnF
        await (0, execa_1.default)('setx', [envVarName, envVarValue]);
    }
}
//# sourceMappingURL=setupOnWindows.js.map